-------------------------------------------------------------------------------
-- CREATE INDEXES TO OPTIMIZE DELTA REFRESH PROCEDURE AND ASSESSMENT FILTERS --
-------------------------------------------------------------------------------

DROP INDEX CFAUDIT.ASSESSMENT_RESULTS_TIMESTAMP_IDX;
CREATE INDEX CFAUDIT.ASSESSMENT_RESULTS_TIMESTAMP_IDX ON CFAUDIT.ASSESSMENT_RESULTS (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;
DROP INDEX CFFACT.T_ACCOUNT_TIMESTAMP_IDX;
CREATE INDEX CFFACT.T_ACCOUNT_TIMESTAMP_IDX ON CFFACT.T_ACCOUNT (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;
DROP INDEX CFFACT.T_EVENT_TIMESTAMP_IDX;
CREATE INDEX CFFACT.T_EVENT_TIMESTAMP_IDX ON CFFACT.T_EVENT (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;
DROP INDEX CFFACT.T_PARTY_TIMESTAMP_IDX;
CREATE INDEX CFFACT.T_PARTY_TIMESTAMP_IDX ON CFFACT.T_PARTY (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;
DROP INDEX CFFACT.T_PHYSICAL_OBJECT_TIMESTAMP_IDX;
CREATE INDEX CFFACT.T_PHYSICAL_OBJECT_TIMESTAMP_IDX ON CFFACT.T_PHYSICAL_OBJECT (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;
DROP INDEX CFFACT.T_TRANSACTION_RECORDS_TIMESTAMP_IDX;
CREATE INDEX CFFACT.T_TRANSACTION_RECORDS_TIMESTAMP_IDX ON CFFACT.T_TRANSACTION_RECORDS (TIMESTAMP ASC) MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COLLECT SAMPLED DETAILED STATISTICS COMPRESS NO;

----------------------------------------------------
-- CREATE CFM_OBJECT TABLE AND ASSOCIATED INDEXES --
----------------------------------------------------

-- Create CFM_OBJECT table
DROP TABLE CFFACT.CFM_OBJECT;
CREATE TABLE CFFACT.CFM_OBJECT (
		"SYSTEM_ID" BIGINT NOT NULL, 
		"OBJECT_ID" BIGINT NOT NULL, 
		"OBJECT_NAME" VARCHAR(200), 
		"OBJECT_DESCRIPTION" VARCHAR(2000), 
		"OBJECT_DEFAULT_ASSESSED_VALUE" DECIMAL(20, 3), 
		"OBJECT_TYPE_ID" BIGINT NOT NULL,
		"OBJECT_TYPE_NAME" VARCHAR(100), 
		"OBJECT_TYPE_DESCRIPTION" VARCHAR(2000), 
		"OBJECT_TYPE_STEREOTYPE" VARCHAR(20), 
		"OBJECT_SUBTYPE_ID" BIGINT, 
		"OBJECT_SUBTYPE_NAME" VARCHAR(100), 
		"OBJECT_SUBTYPE_DESCRIPTION" VARCHAR(2000), 
		"OBJECT_SUBTYPE_STEREOTYPE" VARCHAR(20)
	)
	ORGANIZE BY ROW
	DATA CAPTURE NONE 
	IN "USERSPACE1"
	COMPRESS YES;
	
-- Create clustered index before primary key
CREATE INDEX CFFACT.CFM_OBJECT_OBJECT_ID_IDX ON CFFACT.CFM_OBJECT (OBJECT_ID ASC, OBJECT_TYPE_ID ASC, SYSTEM_ID ASC) CLUSTER MINPCTUSED 0 ALLOW REVERSE SCANS PAGE SPLIT SYMMETRIC COMPRESS YES;
-- Create Primary key (will use existing cluster index)
ALTER TABLE CFFACT.CFM_OBJECT ADD CONSTRAINT "CFM_OBJECT_PK" PRIMARY KEY ("OBJECT_ID", "OBJECT_TYPE_ID", "SYSTEM_ID");

-----------------------------------------------------------------------------------------------------
-- CREATE CFM_OBJECT_REFRESH TABLE TO TRACK STATISTICS EACH TIME THE REFRESH PROCEDURES ARE CALLED --
-----------------------------------------------------------------------------------------------------

DROP TABLE CFAUDIT.CFM_OBJECT_REFRESH;
CREATE TABLE CFAUDIT.CFM_OBJECT_REFRESH (
		"ID" BIGINT NOT NULL GENERATED BY DEFAULT AS IDENTITY ( START WITH 1 INCREMENT BY 1 MINVALUE -9223372036854775808 MAXVALUE 9223372036854775807 NO CYCLE NO CACHE NO ORDER ), 
		"ROWS_READ" BIGINT, 
		"ROWS_SKIPPED" BIGINT, 
		"ROWS_LOADED" BIGINT, 
		"ROWS_REJECTED" BIGINT, 
		"ROWS_DELETED" BIGINT, 
		"ROWS_COMMITTED" BIGINT, 
		"CALLER" VARCHAR(50),
		"UPDATE_TIME" TIMESTAMP NOT NULL GENERATED ALWAYS FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP , 
		"MSG_RETRIEVAL" VARCHAR(500),
		"MSG_REMOVAL" VARCHAR(500)
	)
	ORGANIZE BY ROW
	DATA CAPTURE NONE 
	IN "USERSPACE1"
	COMPRESS YES;

------------------------------------------------------------------------------
-- SET UP SYSTEM PROPERTY TO KEEP TRACK OF WHEN CFM_OBJECT WAS LAST UPDATED --
------------------------------------------------------------------------------
	
-- Currently there is no constraint to stop duplicates in SYSTEM_PROPS, so we delete first in case this script is re-run
delete from cfconfig.system_props where stereotype='CF.DM.COGNOS.CFM_OBJECT.LASTUPDATETIME';
-- Add new system property to keep track of the last update time for each CFM object
insert into cfconfig.system_props (NAME,DESCRIPTION,VALUE,STEREOTYPE) values ('Last update time for CFM_OBJECT table','Keeps track of the last update time of the table CFM_OBJECT', '1970-01-01 00:00:00','CF.DM.COGNOS.CFM_OBJECT.LASTUPDATETIME');

----------------------------------------------
-- CREATE DELTA AND FULL REFRESH PROCEDURES --
----------------------------------------------
	
-- Create procedure to do a delta refresh of CFM_OBJECT
--#SET TERMINATOR @
CREATE OR replace PROCEDURE cffact.refresh_cfm_object_delta
language sql
L1: begin
   DECLARE stmt VARCHAR(20000);
   DECLARE cfm_object_lastupdatetime VARCHAR(50);
   DECLARE rows_read             INTEGER;
   DECLARE rows_skipped          INTEGER;
   DECLARE rows_loaded           INTEGER;
   DECLARE rows_rejected         INTEGER;
   DECLARE rows_deleted          INTEGER;
   DECLARE rows_committed        INTEGER;
   DECLARE rows_partitioned      INTEGER;
   DECLARE num_agentinfo_entries INTEGER;
   DECLARE msg_retrieval         VARCHAR(500);
   DECLARE msg_removal           VARCHAR(500);
   DECLARE res RESULT_SET_LOCATOR varying;

   SELECT value into cfm_object_lastupdatetime from cfconfig.system_props where stereotype='CF.DM.COGNOS.CFM_OBJECT.LASTUPDATETIME';
   update cfconfig.system_props set value=to_char(current timestamp,'YYYY-MM-DD HH24:MI:SS') where stereotype='CF.DM.COGNOS.CFM_OBJECT.LASTUPDATETIME';
   
   set stmt = '(SELECT account.account_id_sys_id AS SYSTEM_ID, 
       account.account_id AS OBJECT_ID, 
       account.NAME AS OBJECT_NAME, 
       account.description AS OBJECT_DESCRIPTION, 
       COALESCE(account_value_assessment.assessed_value, 0.00) AS  OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME) AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description)  AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype AS OBJECT_TYPE_STEREOTYPE, 
       account.account_type_id AS OBJECT_SUBTYPE_ID, 
       COALESCE (ACCOUNT_TYPE_MULTILANG.NAME, account_type.NAME) AS OBJECT_SUBTYPE_NAME, 
       COALESCE (ACCOUNT_TYPE_MULTILANG.description, account_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       account_type.stereotype AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.account AS ACCOUNT 
       INNER JOIN cffact.account_type AS ACCOUNT_TYPE 
               ON account.account_type_id = account_type.account_type_id 
       LEFT OUTER JOIN ( cffact.account_value_assessment AS ACCOUNT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = account_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON account.account_id = account_value_assessment.account_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE  ON object_type.stereotype = ''account'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS ACCOUNT_TYPE_MULTILANG 
                    ON account_type.account_type_id = ACCOUNT_TYPE_MULTILANG.code_id 
                       AND ACCOUNT_TYPE_MULTILANG.source = ''CFFACT.ACCOUNT_TYPE''
		WHERE ACCOUNT.TIMESTAMP > TIMESTAMP(''' || cfm_object_lastupdatetime || ''')

UNION

SELECT event.event_id_sys_id                                                 AS SYSTEM_ID, 
       event.event_id                                                        AS OBJECT_ID, 
       event.NAME                                                            AS OBJECT_NAME, 
       event.description                                                     AS OBJECT_DESCRIPTION, 
       COALESCE(event_value_assessment.assessed_value, 0.00)                 AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       event.event_type_id                                                   AS OBJECT_SUBTYPE_ID, 
       COALESCE (EVENT_TYPE_MULTILANG.NAME, event_type.NAME)                 AS OBJECT_SUBTYPE_NAME, 
       COALESCE (EVENT_TYPE_MULTILANG.description, event_type.description)   AS OBJECT_SUBTYPE_DESCRIPTION, 
       event_type.stereotype                                                 AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.event AS EVENT 
       INNER JOIN cffact.event_type AS EVENT_TYPE 
               ON event.event_type_id = event_type.event_type_id 
       LEFT OUTER JOIN ( cffact.event_value_assessment AS EVENT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                                 ON  value_assessment_type.value_assessment_type_id = event_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON event.event_id = event_value_assessment.event_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE ON object_type.stereotype = ''event'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME,
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS EVENT_TYPE_MULTILANG 
                    ON event_type.event_type_id = EVENT_TYPE_MULTILANG.code_id 
                       AND EVENT_TYPE_MULTILANG.source = ''CFFACT.EVENT_TYPE''
		WHERE EVENT.TIMESTAMP > TIMESTAMP(''' || cfm_object_lastupdatetime || ''')

UNION

SELECT party.party_id_sys_id                                                 AS SYSTEM_ID, 
       party.party_id                                                        AS OBJECT_ID, 
       COALESCE(party_name.text, '''')                                       AS OBJECT_NAME, 
       COALESCE(party_name.text, '''')                                       AS OBJECT_DESCRIPTION, 
       COALESCE(party_value_assessment.assessed_value, 0.00)                 AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       object_type.object_type_id                                            AS OBJECT_SUBTYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_SUBTYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.party AS PARTY 
       LEFT OUTER JOIN ( cffact.party_name AS PARTY_NAME 
                         INNER JOIN cffact.party_name_type AS PARTY_NAME_TYPE 
                                 ON party_name.party_name_type_id = 
                                    party_name_type.party_name_type_id 
                                    AND party_name_type.stereotype = ''legal'' ) 
                    ON party_name.party_id = party.party_id 
       LEFT OUTER JOIN ( cffact.party_value_assessment AS PARTY_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS  VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = party_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON party.party_id = party_value_assessment.party_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE 
               ON object_type.stereotype = ''party''
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE''
		WHERE PARTY.TIMESTAMP > TIMESTAMP(''' || cfm_object_lastupdatetime || ''')

UNION

SELECT physical_object.object_id_sys_id AS SYSTEM_ID, 
       physical_object.object_id AS OBJECT_ID, 
       physical_object.NAME AS OBJECT_NAME, 
       physical_object.description AS OBJECT_DESCRIPTION, 
       COALESCE(physical_object_value_assessment.assessed_value, 0.00) AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME) AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description)  AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype AS OBJECT_TYPE_STEREOTYPE, 
       physical_object.physical_object_type_id AS OBJECT_SUBTYPE_ID, 
       COALESCE (PHYSICAL_OBJECT_TYPE_MULTILANG.NAME, physical_object_type.NAME) AS OBJECT_SUBTYPE_NAME, 
       COALESCE (PHYSICAL_OBJECT_TYPE_MULTILANG.description, physical_object_type.description)   AS OBJECT_SUBTYPE_DESCRIPTION,
       physical_object_type.stereotype  AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.physical_object AS PHYSICAL_OBJECT 
       INNER JOIN cffact.physical_object_type AS PHYSICAL_OBJECT_TYPE 
               ON physical_object.physical_object_type_id = physical_object_type.physical_object_type_id 
       LEFT OUTER JOIN ( cffact.physical_object_value_assessment AS PHYSICAL_OBJECT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                             ON value_assessment_type.value_assessment_type_id = physical_object_value_assessment.value_assessment_type_id 
                        AND value_assessment_type.stereotype = ''default'' ) 
             ON physical_object.object_id = physical_object_value_assessment.object_id 
        INNER JOIN cffact.object_type AS OBJECT_TYPE 
           ON object_type.stereotype = ''physical_object'' 
        LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                        Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                        Cast(NULL AS VARCHAR(50))   AS NAME, 
                        Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                        Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
             ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
        LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                        Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                        Cast(NULL AS VARCHAR(50))   AS NAME, 
                        Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                        Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                 FROM   cfconfig.registered_lang 
                 WHERE  1 = 2) AS PHYSICAL_OBJECT_TYPE_MULTILANG 
             ON physical_object_type.physical_object_type_id = PHYSICAL_OBJECT_TYPE_MULTILANG.code_id 
                AND PHYSICAL_OBJECT_TYPE_MULTILANG.source = ''CFFACT.PHYSICAL_OBJECT_TYPE''
		WHERE PHYSICAL_OBJECT.TIMESTAMP > TIMESTAMP(''' || cfm_object_lastupdatetime || ''')

UNION

SELECT TRANSACTION.transaction_id_sys_id                                     AS SYSTEM_ID, 
       TRANSACTION.transaction_id                                            AS OBJECT_ID, 
       TRANSACTION.transaction_reference                                     AS OBJECT_NAME, 
       TRANSACTION.description                                               AS OBJECT_DESCRIPTION, 
       COALESCE(transaction_value_assessment.assessed_value, 0.00)           AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       TRANSACTION.transaction_type_id                                       AS OBJECT_SUBTYPE_ID, 
       COALESCE (TRANSACTION_TYPE_MULTILANG.NAME, transaction_type.NAME)     AS OBJECT_SUBTYPE_NAME, 
       COALESCE (TRANSACTION_TYPE_MULTILANG.description,transaction_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       transaction_type.stereotype                                           AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.transaction_records AS TRANSACTION 
       INNER JOIN cffact.transaction_type AS TRANSACTION_TYPE 
               ON TRANSACTION.transaction_type_id = transaction_type.transaction_type_id 
       LEFT OUTER JOIN ( cffact.transaction_value_assessment AS TRANSACTION_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = transaction_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON TRANSACTION.transaction_id = transaction_value_assessment.transaction_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE 
               ON object_type.stereotype = ''transaction'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS TRANSACTION_TYPE_MULTILANG 
                    ON transaction_type.transaction_type_id = TRANSACTION_TYPE_MULTILANG.code_id 
                       AND TRANSACTION_TYPE_MULTILANG.source = ''CFFACT.TRANSACTION_TYPE''
		WHERE TRANSACTION.TIMESTAMP > TIMESTAMP(''' || cfm_object_lastupdatetime || ''')
)';
   
   CALL sysproc.admin_cmd( 'LOAD FROM ' || stmt || ' OF CURSOR MESSAGES ON SERVER INSERT INTO CFFACT.CFM_OBJECT 
                            INDEXING MODE INCREMENTAL ALLOW READ ACCESS' );
   ASSOCIATE RESULT SET locators(res) WITH PROCEDURE sysproc.admin_cmd;
   ALLOCATE rscur CURSOR FOR RESULT SET res;
   FETCH rscur INTO rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,rows_partitioned,num_agentinfo_entries,
                    msg_retrieval,msg_removal;
   insert into cfaudit.cfm_object_refresh (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,caller,msg_retrieval,msg_removal) values
                  (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,'cffact.refresh_cfm_object_delta',msg_retrieval,msg_removal);
END l1@
--#SET TERMINATOR ; 


-- Create procedure to do a full refresh of CFM_OBJECT
--#SET TERMINATOR @
CREATE OR replace PROCEDURE cffact.refresh_cfm_object_full
language sql
L1: begin
   DECLARE stmt VARCHAR(20000);
   DECLARE rows_read             INTEGER;
   DECLARE rows_skipped          INTEGER;
   DECLARE rows_loaded           INTEGER;
   DECLARE rows_rejected         INTEGER;
   DECLARE rows_deleted          INTEGER;
   DECLARE rows_committed        INTEGER;
   DECLARE rows_partitioned      INTEGER;
   DECLARE num_agentinfo_entries INTEGER;
   DECLARE msg_retrieval         VARCHAR(500);
   DECLARE msg_removal           VARCHAR(500);
   DECLARE res RESULT_SET_LOCATOR varying;

   update cfconfig.system_props set value=to_char(current timestamp,'YYYY-MM-DD HH24:MI:SS') where stereotype='CF.DM.COGNOS.CFM_OBJECT.LASTUPDATETIME';
   
   set stmt = '(SELECT account.account_id_sys_id AS SYSTEM_ID, 
       account.account_id AS OBJECT_ID, 
       account.NAME AS OBJECT_NAME, 
       account.description AS OBJECT_DESCRIPTION, 
       COALESCE(account_value_assessment.assessed_value, 0.00) AS  OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME) AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description)  AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype AS OBJECT_TYPE_STEREOTYPE, 
       account.account_type_id AS OBJECT_SUBTYPE_ID, 
       COALESCE (ACCOUNT_TYPE_MULTILANG.NAME, account_type.NAME) AS OBJECT_SUBTYPE_NAME, 
       COALESCE (ACCOUNT_TYPE_MULTILANG.description, account_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       account_type.stereotype AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.account AS ACCOUNT 
       INNER JOIN cffact.account_type AS ACCOUNT_TYPE 
               ON account.account_type_id = account_type.account_type_id 
       LEFT OUTER JOIN ( cffact.account_value_assessment AS ACCOUNT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = account_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON account.account_id = account_value_assessment.account_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE  ON object_type.stereotype = ''account'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS ACCOUNT_TYPE_MULTILANG 
                    ON account_type.account_type_id = ACCOUNT_TYPE_MULTILANG.code_id 
                       AND ACCOUNT_TYPE_MULTILANG.source = ''CFFACT.ACCOUNT_TYPE''

UNION

SELECT event.event_id_sys_id                                                 AS SYSTEM_ID, 
       event.event_id                                                        AS OBJECT_ID, 
       event.NAME                                                            AS OBJECT_NAME, 
       event.description                                                     AS OBJECT_DESCRIPTION, 
       COALESCE(event_value_assessment.assessed_value, 0.00)                 AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       event.event_type_id                                                   AS OBJECT_SUBTYPE_ID, 
       COALESCE (EVENT_TYPE_MULTILANG.NAME, event_type.NAME)                 AS OBJECT_SUBTYPE_NAME, 
       COALESCE (EVENT_TYPE_MULTILANG.description, event_type.description)   AS OBJECT_SUBTYPE_DESCRIPTION, 
       event_type.stereotype                                                 AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.event AS EVENT 
       INNER JOIN cffact.event_type AS EVENT_TYPE 
               ON event.event_type_id = event_type.event_type_id 
       LEFT OUTER JOIN ( cffact.event_value_assessment AS EVENT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                                 ON  value_assessment_type.value_assessment_type_id = event_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON event.event_id = event_value_assessment.event_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE ON object_type.stereotype = ''event'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME,
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS EVENT_TYPE_MULTILANG 
                    ON event_type.event_type_id = EVENT_TYPE_MULTILANG.code_id 
                       AND EVENT_TYPE_MULTILANG.source = ''CFFACT.EVENT_TYPE''

UNION

SELECT party.party_id_sys_id                                                 AS SYSTEM_ID, 
       party.party_id                                                        AS OBJECT_ID, 
       COALESCE(party_name.text, '''')                                       AS OBJECT_NAME, 
       COALESCE(party_name.text, '''')                                       AS OBJECT_DESCRIPTION, 
       COALESCE(party_value_assessment.assessed_value, 0.00)                 AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       object_type.object_type_id                                            AS OBJECT_SUBTYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_SUBTYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.party AS PARTY 
       LEFT OUTER JOIN ( cffact.party_name AS PARTY_NAME 
                         INNER JOIN cffact.party_name_type AS PARTY_NAME_TYPE 
                                 ON party_name.party_name_type_id = 
                                    party_name_type.party_name_type_id 
                                    AND party_name_type.stereotype = ''legal'' ) 
                    ON party_name.party_id = party.party_id 
       LEFT OUTER JOIN ( cffact.party_value_assessment AS PARTY_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS  VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = party_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON party.party_id = party_value_assessment.party_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE 
               ON object_type.stereotype = ''party''
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE''

UNION

SELECT physical_object.object_id_sys_id AS SYSTEM_ID, 
       physical_object.object_id AS OBJECT_ID, 
       physical_object.NAME AS OBJECT_NAME, 
       physical_object.description AS OBJECT_DESCRIPTION, 
       COALESCE(physical_object_value_assessment.assessed_value, 0.00) AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME) AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description)  AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype AS OBJECT_TYPE_STEREOTYPE, 
       physical_object.physical_object_type_id AS OBJECT_SUBTYPE_ID, 
       COALESCE (PHYSICAL_OBJECT_TYPE_MULTILANG.NAME, physical_object_type.NAME) AS OBJECT_SUBTYPE_NAME, 
       COALESCE (PHYSICAL_OBJECT_TYPE_MULTILANG.description, physical_object_type.description)   AS OBJECT_SUBTYPE_DESCRIPTION,
       physical_object_type.stereotype  AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.physical_object AS PHYSICAL_OBJECT 
       INNER JOIN cffact.physical_object_type AS PHYSICAL_OBJECT_TYPE 
               ON physical_object.physical_object_type_id = physical_object_type.physical_object_type_id 
       LEFT OUTER JOIN ( cffact.physical_object_value_assessment AS PHYSICAL_OBJECT_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                             ON value_assessment_type.value_assessment_type_id = physical_object_value_assessment.value_assessment_type_id 
                        AND value_assessment_type.stereotype = ''default'' ) 
             ON physical_object.object_id = physical_object_value_assessment.object_id 
        INNER JOIN cffact.object_type AS OBJECT_TYPE 
           ON object_type.stereotype = ''physical_object'' 
        LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                        Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                        Cast(NULL AS VARCHAR(50))   AS NAME, 
                        Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                        Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
             ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
        LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                        Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                        Cast(NULL AS VARCHAR(50))   AS NAME, 
                        Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                        Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                 FROM   cfconfig.registered_lang 
                 WHERE  1 = 2) AS PHYSICAL_OBJECT_TYPE_MULTILANG 
             ON physical_object_type.physical_object_type_id = PHYSICAL_OBJECT_TYPE_MULTILANG.code_id 
                AND PHYSICAL_OBJECT_TYPE_MULTILANG.source = ''CFFACT.PHYSICAL_OBJECT_TYPE''

UNION

SELECT TRANSACTION.transaction_id_sys_id                                     AS SYSTEM_ID, 
       TRANSACTION.transaction_id                                            AS OBJECT_ID, 
       TRANSACTION.transaction_reference                                     AS OBJECT_NAME, 
       TRANSACTION.description                                               AS OBJECT_DESCRIPTION, 
       COALESCE(transaction_value_assessment.assessed_value, 0.00)           AS OBJECT_DEFAULT_ASSESSED_VALUE, 
       object_type.object_type_id                                            AS OBJECT_TYPE_ID, 
       COALESCE (OBJECT_TYPE_MULTILANG.NAME, object_type.NAME)               AS OBJECT_TYPE_NAME, 
       COALESCE (OBJECT_TYPE_MULTILANG.description, object_type.description) AS OBJECT_TYPE_DESCRIPTION, 
       object_type.stereotype                                                AS OBJECT_TYPE_STEREOTYPE, 
       TRANSACTION.transaction_type_id                                       AS OBJECT_SUBTYPE_ID, 
       COALESCE (TRANSACTION_TYPE_MULTILANG.NAME, transaction_type.NAME)     AS OBJECT_SUBTYPE_NAME, 
       COALESCE (TRANSACTION_TYPE_MULTILANG.description,transaction_type.description) AS OBJECT_SUBTYPE_DESCRIPTION, 
       transaction_type.stereotype                                           AS OBJECT_SUBTYPE_STEREOTYPE 
FROM   cffact.transaction_records AS TRANSACTION 
       INNER JOIN cffact.transaction_type AS TRANSACTION_TYPE 
               ON TRANSACTION.transaction_type_id = transaction_type.transaction_type_id 
       LEFT OUTER JOIN ( cffact.transaction_value_assessment AS TRANSACTION_VALUE_ASSESSMENT 
                         INNER JOIN cffact.value_assessment_type AS VALUE_ASSESSMENT_TYPE 
                         ON value_assessment_type.value_assessment_type_id = transaction_value_assessment.value_assessment_type_id 
                         AND value_assessment_type.stereotype = ''default'' ) 
                    ON TRANSACTION.transaction_id = transaction_value_assessment.transaction_id 
       INNER JOIN cffact.object_type AS OBJECT_TYPE 
               ON object_type.stereotype = ''transaction'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS OBJECT_TYPE_MULTILANG 
                    ON object_type.object_type_id = OBJECT_TYPE_MULTILANG.code_id 
                       AND OBJECT_TYPE_MULTILANG.source = ''CFFACT.OBJECT_TYPE'' 
       LEFT OUTER JOIN (SELECT Cast(NULL AS BIGINT)        AS CODE_ID, 
                               Cast(NULL AS VARCHAR(100))  AS SOURCE, 
                               Cast(NULL AS VARCHAR(50))   AS NAME, 
                               Cast(NULL AS VARCHAR(2000)) AS DESCRIPTION, 
                               Cast(NULL AS BIGINT)        AS TYPECODE_ID 
                        FROM   cfconfig.registered_lang 
                        WHERE  1 = 2) AS TRANSACTION_TYPE_MULTILANG 
                    ON transaction_type.transaction_type_id = TRANSACTION_TYPE_MULTILANG.code_id 
                       AND TRANSACTION_TYPE_MULTILANG.source = ''CFFACT.TRANSACTION_TYPE''
)';
   
   CALL sysproc.admin_cmd( 'LOAD FROM ' || stmt || ' OF CURSOR MESSAGES ON SERVER REPLACE INTO CFFACT.CFM_OBJECT 
                            INDEXING MODE REBUILD ALLOW NO ACCESS' );
   ASSOCIATE RESULT SET locators(res) WITH PROCEDURE sysproc.admin_cmd;
   ALLOCATE rscur CURSOR FOR RESULT SET res;
   FETCH rscur INTO rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,rows_partitioned,num_agentinfo_entries,
                    msg_retrieval,msg_removal;
   insert into cfaudit.cfm_object_refresh (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,caller,msg_retrieval,msg_removal) values
                  (rows_read,rows_skipped,rows_loaded,rows_rejected,rows_deleted,rows_committed,'cffact.refresh_cfm_object_full',msg_retrieval,msg_removal);
END l1@
--#SET TERMINATOR ; 

-- Call full refresh procedure to complete initial population of table
CALL CFFACT.REFRESH_CFM_OBJECT_FULL;

----------------------------------------
-- UPDATE STATISTICS ON OBJECT TABLES --
----------------------------------------

RUNSTATS ON TABLE CFFACT.T_ACCOUNT ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
RUNSTATS ON TABLE CFFACT.T_EVENT ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
RUNSTATS ON TABLE CFFACT.T_PARTY ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
RUNSTATS ON TABLE CFFACT.T_PHYSICAL_OBJECT ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
RUNSTATS ON TABLE CFFACT.T_TRANSACTION_RECORDS ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;
RUNSTATS ON TABLE CFFACT.CFM_OBJECT ON ALL COLUMNS WITH DISTRIBUTION ON ALL COLUMNS AND DETAILED INDEXES ALL;

-------------------------------------------------------------------------
-- SET UP SCHEDULED TASKS TO DO DELTA AND FULL REFRESHES OF CFM_OBJECT --
-------------------------------------------------------------------------

-- Create tablespaces for tools (if these already exist, the resulting warnings can be ignored)
CREATE LARGE TABLESPACE "SYSTOOLSPACE"
	IN DATABASE PARTITION GROUP "IBMCATGROUP"
	PAGESIZE 32 K
	MANAGED BY AUTOMATIC STORAGE
	AUTORESIZE YES
	EXTENTSIZE 4
	BUFFERPOOL "IBMDEFAULTBP"
	OVERHEAD INHERIT
	TRANSFERRATE INHERIT
	DATA TAG INHERIT
	USING STOGROUP "IBMSTOGROUP"
	NO FILE SYSTEM CACHING;

CREATE USER TEMPORARY TABLESPACE "SYSTOOLSTMPSPACE"
	IN DATABASE PARTITION GROUP "IBMCATGROUP"
	PAGESIZE 32 K
	MANAGED BY AUTOMATIC STORAGE
	EXTENTSIZE 4
	BUFFERPOOL "IBMDEFAULTBP"
	OVERHEAD INHERIT
	TRANSFERRATE INHERIT
	USING STOGROUP "IBMSTOGROUP"
	FILE SYSTEM CACHING;

GRANT USE OF TABLESPACE "SYSTOOLSTMPSPACE" TO PUBLIC;

-- Note that the following command must be run first from the command line to enable the task scheduler
--db2set DB2_ATS_ENABLE=YES
--define delta task to run once every 30 mins
CALL SYSPROC.ADMIN_TASK_REMOVE('REFRESH CFM_OBJECT DELTA',NULL);
CALL SYSPROC.ADMIN_TASK_ADD('REFRESH CFM_OBJECT DELTA',CURRENT_TIMESTAMP,NULL,NULL,'*/30 * * * *','CFFACT','REFRESH_CFM_OBJECT_DELTA',NULL,NULL,NULL);
--define full task to run once a week
CALL SYSPROC.ADMIN_TASK_REMOVE('REFRESH CFM_OBJECT FULL',NULL);
CALL SYSPROC.ADMIN_TASK_ADD('REFRESH CFM_OBJECT FULL',CURRENT_TIMESTAMP,NULL,NULL,'0 6 * * 0','CFFACT','REFRESH_CFM_OBJECT_FULL',NULL,NULL,NULL);
-- To update the task, e.g. to change the schedule to run every 5 mins
-- CALL SYSPROC.ADMIN_TASK_UPDATE('REFRESH CFM_OBJECT DELTA',CURRENT_TIMESTAMP,NULL,NULL,'*/5 * * * *',NULL,NULL);
-- To delete a task: CALL SYSPROC.ADMIN_TASK_REMOVE('REFRESH CFM_OBJECT DELTA',NULL)
-- To check the status of a refresh task: SELECT * from SYSTOOLS.ADMIN_TASK_STATUS WHERE NAME like 'REFRESH CFM_OBJECT%'
-- To check what tasks have been defined: select * from systools.admin_task_list
